"""
Identifier:     csst_common/file.py
Name:           file.py
Description:    file operation
Author:         Bo Zhang
Created:        2023-12-13
Modified-History:
    2023-12-13, Bo Zhang, created
    2023-12-13, Bo Zhang, add File
    2023-12-15, Bo Zhang, use re to match file name
    2023-12-15, Bo Zhang, add module header
"""
import os
import re
from typing import Optional


class File:
    """
    CSST MSC standard file path parser.

    It can be used for parse CSST MSC file paths and generate corresponding output file paths.

    Parameters
    ----------
    file_path : str
        File path.
    new_dir : str
        New directory.
    """

    def __init__(
        self, file_path: str = "/path/to/file.fits", new_dir: Optional[str] = None
    ):
        self.file_path = file_path
        # get dir name
        self.dirname = os.path.dirname(self.file_path)
        # get file name
        self.file_name = os.path.basename(self.file_path)
        # default new dir
        self.new_dir = new_dir if new_dir is not None else self.dirname

        # match file name
        pattern = re.compile(
            r"(?P<telescope>[A-Z]{4})_"
            r"(?P<instrument>[A-Z]+)_"
            r"(?P<project>[A-Z]+)_"
            r"(?P<obs_type>[A-Z]+)_"
            r"(?P<exp_start>[0-9]{14})_"
            r"(?P<exp_stop>[0-9]{14})_"
            r"(?P<obs_id>[0-9]{11,14})_"
            r"(?P<detector>[0-9]+)_"
            r"L(?P<level>[0-9]{1})_"
            r"V(?P<version>[0-9]{2})"
            r"(?P<ext>[a-z.]+)"
        )
        self.mo = re.fullmatch(pattern, self.file_name)
        assert self.mo is not None, f"Pattern does not match for file: {self.file_name}"
        # set attributes
        for k, v in self.mo.groupdict().items():
            self.__setattr__(k, v)

    def derive(
        self,
        new_dir=None,
        telescope=None,
        instrument=None,
        project=None,
        obs_type=None,
        exp_start=None,
        exp_stop=None,
        obs_id=None,
        detector=None,
        level=None,
        version=None,
        ext=None,
    ):
        return os.path.join(
            new_dir if new_dir is not None else self.new_dir,
            f"{self.telescope if telescope is None else str(telescope)}_"
            f"{self.instrument if instrument is None else str(instrument)}_"
            f"{self.project if project is None else str(project)}_"
            f"{self.obs_type if obs_type is None else str(obs_type)}_"
            f"{self.exp_start if exp_start is None else str(exp_start)}_"
            f"{self.exp_stop if exp_stop is None else str(exp_stop)}_"
            f"{self.obs_id if obs_id is None else str(obs_id)}_"
            f"{self.detector if detector is None else str(detector)}_"
            f"L{self.level if level is None else str(level)}_"
            f"V{self.version if version is None else str(version)}"
            f"{self.ext if ext is None else str(ext)}",
        )

    def derive0(self, *args, **kwargs):
        return self.derive(*args, **kwargs, level=0)

    def derive1(self, *args, **kwargs):
        return self.derive(*args, **kwargs, level=1)

    def derive2(self, *args, **kwargs):
        return self.derive(*args, **kwargs, level=2)

    def replace_ext(
        self, new_ext: Optional[str] = "img.fits", new_dir: Optional[str] = None
    ) -> str:
        if new_dir is None:
            new_dir = self.new_dir
        if new_ext is None:
            new_ext = self.ext

        if new_ext.startswith("."):
            return os.path.join(new_dir, self.prefix + new_ext)
        else:
            return os.path.join(new_dir, self.prefix + "_" + new_ext)

    def __repr__(self):
        return f"<File {self.file_path}>"
