"""
Identifier:     csst_common/io.py
Name:           io.py
Description:    IO module
Author:         Bo Zhang
Created:        2023-12-13
Modified-History:
    2023-12-10, Bo Zhang, created
    2023-12-15, Bo Zhang, add verify_checksum and append_header, add module header
"""
import warnings
from copy import deepcopy

from astropy.io import fits


def verify_checksum(file_path) -> bool:
    """
    Verify a .fits file via checksum.

    Return True if checksum is good.

    Parameters
    ----------
    file_path : str
        File path.

    References
    ----------
    https://docs.astropy.org/en/stable/io/fits/usage/verification.html#verification-using-the-fits-checksum-keyword-convention
    """
    with warnings.catch_warnings(record=True) as warning_list:
        # file_path = fits.util.get_testdata_filepath('checksum_false.fits')
        with fits.open(file_path, checksum=True):
            pass
        print(warning_list)
    return len(warning_list) == 0


def append_header(
    h1: fits.Header, h2: fits.Header, duplicates: str = "delete"
) -> fits.Header:
    """
    Append h2 to h1.

    Append fits headers, taken into aacount duplicated keywords.

    Parameters
    ----------
    h1 : fits.Header
        Original fits header.
    h2 : fits.Header
        Extended fits header.
    duplicates : str
        The operation for processing duplicates.
        "delete" to delete duplicated keywords in h1 and keep them in h2.
        "update" to update duplicated keywords in h1 and remove them from h2.

    Returns
    -------
    fits.Header
        The combined header.

    References
    ----------
    https://docs.astropy.org/en/stable/io/fits/usage/headers.html#comment-history-and-blank-keywords
    """
    # copy data
    original_h = deepcopy(h1)
    extended_h = deepcopy(h2)
    original_keys = original_h.keys()
    ignored_keys = ["", "COMMENT", "HISTORY"]
    assert duplicates in ("delete", "update")
    if duplicates == "update":
        for card in extended_h.cards:
            if card.keyword not in ignored_keys and card.keyword in original_keys:
                print(f"Update existing key *{card.keyword} in original fits.Header*")
                original_h.set(card.keyword, card.value, card.comment)
                extended_h.remove(card.keyword)
    elif duplicates == "delete":
        for card in extended_h.cards:
            if card.keyword not in ignored_keys and card.keyword in original_keys:
                print(f"Delete existing key *{card.keyword} in original fits.Header*")
                original_h.remove(card.keyword)
    original_h.extend(extended_h, bottom=True)
    return original_h
