"""
Identifier:     csst_common/pipeline.py
Name:           file.py
Description:    pipeline operation
Author:         Bo Zhang
Created:        2023-12-13
Modified-History:
    2023-07-11, Bo Zhang, created
    2023-07-11, Bo Zhang, add Pipeline class
    2023-12-10, Bo Zhang, update Pipeline
    2023-12-15, Bo Zhang, add module header
"""
import json
import os
import subprocess
import warnings
from typing import Any

from astropy import time

from .ccds import CCDS
from .dfs import DFS
from .file import File
from .logger import get_logger


class Pipeline:
    """
    CSST pipeline configuration class.

    It should be used for every pipeline to initialize environment.

    Parameters
    ----------
    dir_input : str
        Input path.
    dir_output : str
        Output path.
    dir_aux : str
        Aux path.
    dfs_root : str
        DFS root path.
    ccds_root : str
        CCDS root path.
    ccds_cache : str
        CCDS cache path.
    filter_warnings : bool
        If `True`, filter warnings.
    dfs : bool
        If `True`, initialize DFS.
    ccds : bool
        If `True`, initialize CCDS.
    **kwargs : Any
        Additional keyword arguments.
    """

    def __init__(
        self,
        dir_input: str = "/pipeline/input",
        dir_output: str = "/pipeline/output",
        dir_aux: str = "/pipeline/aux",
        dfs_root: str = "/dfs_root",
        ccds_root: str = "/ccds_root",
        ccds_cache: str = "/pipeline/ccds_cache",
        filter_warnings: bool = False,
        dfs: bool = True,
        ccds: bool = False,
        **kwargs: Any,
    ):
        # get pipeline information from env vars
        self.pipeline_id = os.getenv("PIPELINE_ID", "-")
        self.build = os.getenv("BUILD", "-")
        self.created = os.getenv("CREATED", "-")

        # set directory information
        self.dir_input = dir_input
        self.dir_output = dir_output
        self.dir_aux = dir_aux
        self.dfs_root = dfs_root
        self.ccds_root = ccds_root
        self.ccds_cache = ccds_cache

        # additional parameters
        self.kwargs = kwargs

        # set logger
        self.pipeline_logger = get_logger(
            name="pipeline logger",
            filename=os.path.join(self.dir_output, "pipeline.log"),
        )
        self.module_logger = get_logger(
            name="module logger",
            filename=os.path.join(self.dir_output, "module.log"),
        )

        # change working directory
        print(f"Change directory to {self.dir_output}")
        os.chdir(self.dir_output)

        # Frequently used files
        self.msg = MessageWriter(os.path.join(self.dir_output, "message.txt"))
        self.tsr = TimestampRecorder(os.path.join(self.dir_output, "timestamp.txt"))
        # self.exit_code = ExitCode(os.path.join(self.dir_output, "exit_code"))
        # self.error_trace = ErrorTrace(os.path.join(self.dir_output, "error_trace"))

        if dfs:
            self.dfs = DFS(n_try=5)
        else:
            self.dfs = None
        if ccds:
            self.ccds = CCDS(
                ccds_root=ccds_root,
                ccds_cache=ccds_cache,
            )
        else:
            self.ccds = None

        if filter_warnings:
            self.filter_warnings()

    def inspect(self):
        print(f"PIPELINE_ID={self.pipeline_id}")
        print(f"BUILD={self.build}")
        print(f"CREATED={self.created}")

    def clean_output(self):
        """Clean output directory."""
        self.clean_directory(self.dir_output)

    @staticmethod
    def clean_directory(d):
        print(f"Clean output directory '{d}'...")
        r = subprocess.run(f"rm -rf {d}/*", shell=True, capture_output=True)
        print("> ", r)
        r.check_returncode()

    def now(self):
        return time.Time.now().isot

    @staticmethod
    def filter_warnings():
        # Suppress all warnings
        warnings.filterwarnings("ignore")

    @staticmethod
    def reset_warnings(self):
        # Reset warning filters
        warnings.resetwarnings()

    def file(self, file_path):
        return File(file_path, new_dir=self.dir_output)

    def new(self, file_name="test.fits"):
        return os.path.join(self.dir_output, file_name)


# class ErrorTrace:
#     """Write error trace to file."""
#
#     def __init__(self, file_path=""):
#         self.file_path = file_path
#
#     def __repr__(self):
#         return f"< ErrorTrace [{self.file_path}] >"
#
#     def write(self, s: str):
#         with open(self.file_path, "w+") as f:
#             f.write(s)


class MessageWriter:
    """Write JSON format messages to file."""

    def __init__(self, file_path=""):
        self.file_path = file_path

    def __repr__(self):
        return f"< Message [{self.file_path}] >"

    def write(self, dlist):
        with open(self.file_path, "w+") as f:
            for d in dlist:
                f.write(self.dict2msg(d) + "\n")

    @staticmethod
    def dict2msg(d):
        m = json.dumps(d).replace(" ", "")
        return m

    @staticmethod
    def msg2dict(m):
        d = json.loads(m)
        return d


# DEPRECATED
# class ExitCode:
#     def __init__(self, file_path=""):
#         self.file_path = file_path
#
#     def __repr__(self):
#         return f"< ExitCode [{self.file_path}] >"
#
#     def truncate(self):
#         with open(self.file_path, "w") as file:
#             file.truncate(0)
#
#     def write(self, code=0):
#         with open(self.file_path, "w+") as f:
#             f.write(str(code))
#         print(f"Exit with code {code} (written to '{self.file_path}')")


class TimestampRecorder:
    def __init__(self, file_path: str = "tsr.txt"):
        """
        TimestampRecorder Class.

        Initialize a TimestampRecorder object anc connect it to `file_path`.

        Parameters
        ----------
        file_path : str
            Time stamp file path.
        """
        self.file_path = file_path

    def __repr__(self):
        return f"< TimestampRecorder [{self.file_path}] >"

    def empty(self):
        """Clean time stamp file."""
        with open(self.file_path, "w") as file:
            file.truncate(0)

    def touch(self):
        """Write a time stamp."""
        with open(self.file_path, "a+") as f:
            f.write(f"{time.Time.now().isot}+00:00\n")
